from pydicom.dataset import Dataset


def get_contour_dict(
    rs_ds: Dataset,
    ) -> dict:
    """
    Get contour dict from rs_ds.
    Args:
        rs_ds (Dataset): rs_ds
    Returns:
        dict: contour_dict
    Limitations:
        1. Only one image is referenced.
        2. Only one contour is referenced.
    Structure of contour_dict:
    {
        roi_number: {
            'color': [R, G, B],  # ROI display color
            'name': str,         # ROI name
            'dcm_contour': {     # Contours organized by image
                sop_instance_uid: {
                    'sop_class_uid': str,  # SOP Class UID of referenced image
                    'contours': [          # List of contour data arrays
                        [x1, y1, z1, x2, y2, z2, ...],  # First contour
                        [x1, y1, z1, x2, y2, z2, ...],  # Second contour
                        ...
                    ]
                }
            }
        }
    }
    """
    contour_dict = {}
    number_name_map = get_roi_number_to_name(rs_ds)
    for roi_contour_sequence in rs_ds.ROIContourSequence:
        roi_number = roi_contour_sequence.ReferencedROINumber
        roi_color = roi_contour_sequence.ROIDisplayColor
        roi_name = number_name_map[roi_number]
        contour_dict[roi_number] = {}
        contour_dict[roi_number]['color'] = roi_color
        contour_dict[roi_number]['name'] = roi_name
        contour_dict[roi_number]['dcm_contour'] = {}
        for contour_sequence in roi_contour_sequence.ContourSequence:
            sop_instance_uid = contour_sequence.ContourImageSequence[0].ReferencedSOPInstanceUID    # TODO ensure only one image is referenced
            sop_class_uid = contour_sequence.ContourImageSequence[0].ReferencedSOPClassUID
            contour_data = contour_sequence.ContourData

            if sop_instance_uid not in contour_dict[roi_number]['dcm_contour']:
                contour_dict[roi_number]['dcm_contour'][sop_instance_uid] = {}
                contour_dict[roi_number]['dcm_contour'][sop_instance_uid]['sop_class_uid'] = sop_class_uid
                contour_dict[roi_number]['dcm_contour'][sop_instance_uid]['contours'] = []

            contour_dict[roi_number]['dcm_contour'][sop_instance_uid]['contours'].append(contour_data)

    for roi_observations_sequence in rs_ds.RTROIObservationsSequence:
        roi_number = roi_observations_sequence.ReferencedROINumber
        contour_dict[roi_number]['interpreted_type'] = getattr(roi_observations_sequence, "RTROIInterpretedType", "NOTAG")

    return contour_dict


def get_roi_number_to_name(
    rs_ds: Dataset,
    ) -> dict:
    """
    Get roi number to name map from rs_ds.
    Args:
        rs_ds (Dataset): rs_ds
    Returns:
        dict: roi_number_to_name
    """
    roi_number_to_name = {}
    for ssroi in rs_ds.StructureSetROISequence:
        roi_number_to_name[ssroi.ROINumber] = ssroi.ROIName
    return roi_number_to_name



"""
A SimpleITK image Builder.
Date: 2025-07-18
Author: higumalu
"""

import numpy as np
import SimpleITK as sitk

from pydicomrt.utils.image_series_loader import sort_ds_list, get_slice_position, load_sorted_image_series, get_slice_directions

class SimpleITKImageBuilder:
    """
    Build a SimpleITK image
    """
    def __init__(self):
        self.volume = None
        self.origin = None
        self.spacing = None
        self.direction = None
        self.image = None

    def set_volume(self, volume: np.ndarray):
        assert len(volume.shape) == 3, "Volume must be a 3D array"
        self.volume = volume

    def set_origin(self, origin: np.ndarray):
        assert origin.shape == (3,), "Origin must be a 3-element vector"
        self.origin = origin

    def set_spacing(self, spacing: np.ndarray):
        assert spacing.shape == (3,), "Spacing must be a 3-element vector"
        self.spacing = spacing

    def set_direction(self, direction: np.ndarray):
        assert direction.shape == (3, 3), "Direction must be a 3x3 matrix"
        self.direction = direction

    def from_ds_list(self, ds_list):
        self.volume, self.origin, self.spacing, self.direction = parse_ds_list(ds_list)
        return self.build()

    def from_ref_sitk_image(self, volume: np.ndarray, ref_image: sitk.Image):
        self.volume = volume
        self.origin = np.array(ref_image.GetOrigin())
        self.spacing = np.array(ref_image.GetSpacing())
        self.direction = np.array(ref_image.GetDirection()).reshape(3, 3)
        return self.build()

    def from_dcms_dir(self, dcms_dir: str):
        reader = sitk.ImageSeriesReader()
        dicom_names = reader.GetGDCMSeriesFileNames(dcms_dir)
        reader.SetFileNames(dicom_names)
        image = reader.Execute()
        self.volume = sitk.GetArrayFromImage(image)
        self.origin = np.array(image.GetOrigin())
        self.spacing = np.array(image.GetSpacing())
        self.direction = np.array(image.GetDirection()).reshape(3, 3)
        return self.build()

    def build(self):
        assert self.volume is not None, "Volume must be set"
        assert self.origin is not None, "Origin must be set"
        assert self.spacing is not None, "Spacing must be set"
        assert self.direction is not None, "Direction must be set"
        self.image = sitk.GetImageFromArray(self.volume)
        self.image.SetOrigin(self.origin)
        self.image.SetSpacing(self.spacing)
        self.image.SetDirection(self.direction.ravel())
        return self.image

def get_spacing_between_slices(series_data):
    if len(series_data) > 1:
        first = get_slice_position(series_data[0])
        last = get_slice_position(series_data[-1])
        return (last - first) / (len(series_data) - 1)
    # Return nonzero value for one slice just to make the transformation matrix invertible
    return 1.0

def parse_ds_list(ds_list):
    """
    Parse a list of DICOM datasets
    Limitations:
        Only works for axial slices.
        Only works for linear transforms.
        Only works for DICOM datasets with ImagePositionPatient and PixelSpacing attributes.
        Only works for CT/MR DICOM datasets.
    Args:
        ds_list (list): List of DICOM datasets.
    Returns:
        tuple: Tuple of image volume, origin, spacing, and direction.
    """
    ds_list = sort_ds_list(ds_list)
    image_volume = np.stack([ds.pixel_array for ds in ds_list], axis=0)
    slope = getattr(ds_list[0], "RescaleSlope", 1)
    intercept = getattr(ds_list[0], "RescaleIntercept", 0)
    image_volume = image_volume * slope + intercept
    slice_spacing = get_spacing_between_slices(ds_list)
    origin = np.array([float(ds_list[0].ImagePositionPatient[0]), float(ds_list[0].ImagePositionPatient[1]), float(ds_list[0].ImagePositionPatient[2])])
    spacing = np.array([float(ds_list[0].PixelSpacing[0]), float(ds_list[0].PixelSpacing[1]), slice_spacing])
    direction = np.array(get_slice_directions(ds_list[0])).reshape(3, 3)
    return image_volume, origin, spacing, direction

def ds_list_to_sitk_image(ds_list):
    """
    Convert a list of DICOM datasets to a SimpleITK image.
    Args:
        ds_list (list): List of DICOM datasets.
    Returns:
        sitk.Image: SimpleITK image.
    Limitations:
        - Only works for axial slices.
    """
    ds_list = sort_ds_list(ds_list)
    image_volume = np.stack([ds.pixel_array for ds in ds_list], axis=0)
    slope = getattr(ds_list[0], "RescaleSlope", 1)
    intercept = getattr(ds_list[0], "RescaleIntercept", 0)
    image_volume = image_volume * slope + intercept
    slice_spacing = get_spacing_between_slices(ds_list)
    spacing = [float(ds_list[0].PixelSpacing[0]), float(ds_list[0].PixelSpacing[1]), slice_spacing]
    origin = [float(ds_list[0].ImagePositionPatient[0]), float(ds_list[0].ImagePositionPatient[1]), float(ds_list[0].ImagePositionPatient[2])]
    direction = np.array(get_slice_directions(ds_list[0])).reshape(3, 3).ravel()
    sitk_image = sitk.GetImageFromArray(image_volume)
    sitk_image.SetOrigin(origin)
    sitk_image.SetSpacing(spacing)
    sitk_image.SetDirection(direction)
    return sitk_image

def resample_to_reference_image(reference_image, source_image, min_val=-1000.0):
    """
    Resample the input image to the reference image space.
    Args:
        input_image (sitk.Image): Input image.
        target_image (sitk.Image): Target image.
    Returns:
        sitk.Image: Input image with synchronized spacing, origin, and direction.
    """
    resample_image = sitk.Resample(
        source_image,
        reference_image,
        sitk.Transform(),
        sitk.sitkLinear,
        min_val,
        reference_image.GetPixelID())
    return resample_image


if __name__ == "__main__":
    ct_path = "example/data/Mirror/CT"
    ct_ds_list = load_sorted_image_series(ct_path)

    ct_builder = SimpleITKImageBuilder()
    ct_builder.from_dcms_dir(ct_path)
    ct_image = ct_builder.build()
    print(ct_image)

    ct_builder.from_ds_list(ct_ds_list)
    ct_image = ct_builder.build()
    print(ct_image)

    ct_volume = np.random.rand(10, 10, 10)
    ct_builder.from_ref_sitk_image(ct_volume, ct_image)
    ct_image = ct_builder.build()
    print(ct_image)

    # ct_ds_list = load_sorted_image_series(ct_path)
    # image = ds_list_to_sitk_image(ct_ds_list)
    # print(image)



import numpy as np
import SimpleITK as sitk


def apply_transform(
    input_image,
    reference_image=None,
    transform=None,
    default_value=0,
    interpolator=sitk.sitkNearestNeighbor,
):
    """
    Transform a volume of structure with the given deformation field.

    Args
        input_image (SimpleITK.Image): The image, to which the transform is applied
        reference_image (SimpleITK.Image): The image will be resampled into this reference space.
        transform (SimpleITK.Transform): The transformation
        default_value: Default (background) value. Defaults to 0.
        interpolator (int, optional): The interpolation order.
                                Available options:
                                    - SimpleITK.sitkNearestNeighbor
                                    - SimpleITK.sitkLinear
                                    - SimpleITK.sitkBSpline
                                Defaults to SimpleITK.sitkNearestNeighbor

    Returns
        (SimpleITK.Image): the transformed image

    """
    original_image_type = input_image.GetPixelID()

    resampler = sitk.ResampleImageFilter()

    if reference_image:
        resampler.SetReferenceImage(reference_image)
    else:
        resampler.SetReferenceImage(input_image)

    if transform:
        resampler.SetTransform(transform)

    resampler.SetDefaultPixelValue(default_value)
    resampler.SetInterpolator(interpolator)

    output_image = resampler.Execute(input_image)
    output_image = sitk.Cast(output_image, original_image_type)

    return output_image

def registration_command_iteration(method):
    """
    Utility function to print information during (rigid, similarity, translation, B-splines)
    registration
    """
    print("{0:3} = {1:10.5f}".format(method.GetOptimizerIteration(), method.GetMetricValue()))


def stage_iteration(method):
    """
    Utility function to print information during stage change in registration
    """
    print(f"Number of parameters = {method.GetInitialTransform().GetNumberOfParameters()}")


def deformable_registration_command_iteration(method):
    """
    Utility function to print information during demons registration
    """
    print("{0:3} = {1:10.5f}".format(method.GetElapsedIterations(), method.GetMetric()))


def control_point_spacing_distance_to_number(image, grid_spacing):
    """
    Convert grid spacing specified in distance to number of control points
    """
    image_spacing = np.array(image.GetSpacing())
    image_size = np.array(image.GetSize())
    number_points = image_size * image_spacing / np.array(grid_spacing)
    return (number_points + 0.5).astype(int)


def smooth_and_resample(
    image,
    isotropic_voxel_size_mm=None,
    shrink_factor=None,
    smoothing_sigma=None,
    interpolator=sitk.sitkLinear,
):
    """
    Args:
        image (SimpleITK.Image): The image we want to resample.
        isotropic_voxel_size_mm (float | None): New voxel size in millimetres
        shrink_factor (list | float): A number greater than one, such that the new image's size is
            original_size/shrink_factor. Can also be specified independently for each
            dimension (sagittal, coronal, axial).
        smoothing_sigma (list | float): Scale for Gaussian smoothing, this is in physical
            (image spacing) units, not pixels. Can also be specified independently for
            each dimension (sagittal, coronal, axial).
    Return:
        SimpleITK.Image: Image which is a result of smoothing the input and then resampling
        it using the specified Gaussian kernel and shrink factor.
    """
    if smoothing_sigma:
        if hasattr(smoothing_sigma, "__iter__"):
            smoothing_variance = [i * i for i in smoothing_sigma]
        else:
            smoothing_variance = (smoothing_sigma ** 2,) * 3

        maximum_kernel_width = int(
            max([8 * j * i for i, j in zip(image.GetSpacing(), smoothing_variance)])
        )

        image = sitk.DiscreteGaussian(image, smoothing_variance, maximum_kernel_width)

    original_spacing = image.GetSpacing()
    original_size = image.GetSize()

    if shrink_factor and isotropic_voxel_size_mm:
        raise AttributeError(
            "Function must be called with either isotropic_voxel_size_mm or "
            "shrink_factor, not both."
        )

    elif isotropic_voxel_size_mm:
        scale_factor = (
            isotropic_voxel_size_mm * np.ones_like(image.GetSize()) / np.array(image.GetSpacing())
        )
        new_size = [int(sz / float(sf) + 0.5) for sz, sf in zip(original_size, scale_factor)]

    elif shrink_factor:
        if isinstance(shrink_factor, list):
            new_size = [int(sz / float(sf) + 0.5) for sz, sf in zip(original_size, shrink_factor)]
        else:
            new_size = [int(sz / float(shrink_factor) + 0.5) for sz in original_size]

    else:
        return image

    new_spacing = [
        ((size_o_i - 1) * spacing_o_i) / (size_n_i - 1)
        for size_o_i, spacing_o_i, size_n_i in zip(original_size, original_spacing, new_size)
    ]

    return sitk.Resample(
        image,
        new_size,
        sitk.Transform(),
        interpolator,
        image.GetOrigin(),
        new_spacing,
        image.GetDirection(),
        0.0,
        image.GetPixelID(),
    )


def bspline_registration(
    fixed_image,
    moving_image,
    fixed_structure=False,
    moving_structure=False,
    resolution_staging=[8, 4, 2],
    smooth_sigmas=[4, 2, 1],
    sampling_rate=0.1,
    optimiser="LBFGS",
    metric="mean_squares",
    initial_grid_spacing=64,
    grid_scale_factors=[1, 2, 4],
    interp_order=sitk.sitkBSpline,
    default_value=-1000,
    number_of_iterations=20,
    isotropic_resample=False,
    initial_isotropic_size=1,
    number_of_histogram_bins_mi=30,
    verbose=False,
    ncores=8,
):
    """
    B-Spline image registration using ITK

    IMPORTANT - THIS IS UNDER ACTIVE DEVELOPMENT

    Args:
        fixed_image ([SimpleITK.Image]): The fixed (target/primary) image.
        moving_image ([SimpleITK.Image]): The moving (secondary) image.
        fixed_structure (bool, optional): If defined, a binary SimpleITK.Image used to mask metric
                                          evaluation for the moving image. Defaults to False.
        moving_structure (bool, optional): If defined, a binary SimpleITK.Image used to mask metric
                                           evaluation for the fixed image. Defaults to False.
        resolution_staging (list, optional): The multi-resolution downsampling factors.
                                             Defaults to [8, 4, 2].
        smooth_sigmas (list, optional): The multi-resolution smoothing kernel scale (Gaussian).
                                        Defaults to [4, 2, 1].
        sampling_rate (float, optional): The fraction of voxels sampled during each iteration.
                                         Defaults to 0.1.
        optimiser (str, optional): The optimiser algorithm used for image registration.
                                   Available options:
                                    - LBFSGS
                                      (limited-memory Broyden–Fletcher–Goldfarb–Shanno (bounded).)
                                    - LBFSG
                                      (limited-memory Broyden–Fletcher–Goldfarb–Shanno
                                      (unbounded).)
                                    - CGLS (conjugate gradient line search)
                                    - gradient_descent
                                    - gradient_descent_line_search
                                   Defaults to "LBFGS".
        metric (str, optional): The metric to be optimised during image registration.
                                Available options:
                                 - correlation
                                 - mean_squares
                                 - demons
                                 - mutual_information
                                   (used with parameter number_of_histogram_bins_mi)
                                Defaults to "mean_squares".
        initial_grid_spacing (int, optional): Grid spacing of lower resolution stage (in mm).
                                              Defaults to 64.
        grid_scale_factors (list, optional): Factors to determine grid spacing at each
                                             multiresolution stage.
                                             Defaults to [1, 2, 4].
        interp_order (int, optional): Interpolation order of final resampling.
                                      Defaults to sitk.sitkBSpline (cubic).
        default_value (int, optional): Default image value. Defaults to -1000.
        number_of_iterations (int, optional): Number of iterations at each resolution stage.
                                              Defaults to 20.
        isotropic_resample (bool, optional): Flag whether to resample to isotropic resampling
                                             prior to registration.
                                             Defaults to False.
        initial_isotropic_size (int, optional): Voxel size (in mm) of resampled isotropic image
                                                (if used). Defaults to 1.
        number_of_histogram_bins_mi (int, optional): Number of histogram bins used when calculating
                                                     mutual information. Defaults to 30.
        verbose (bool, optional): Print image registration process information. Defaults to False.
        ncores (int, optional): Number of CPU cores used. Defaults to 8.

    Returns:
        [SimpleITK.Image]: The registered moving (secondary) image.
        [SimleITK.Transform]: The linear transformation.

    Notes:
     - smooth_sigmas are relative to resolution staging
        e.g. for image spacing of 1x1x1 mm^3, with smooth sigma=2 and resolution_staging=4, the
        scale of the Gaussian filter would be 2x4 = 8mm (i.e. 8x8x8 mm^3)

    """

    # Re-cast input images
    fixed_image = sitk.Cast(fixed_image, sitk.sitkFloat32)

    moving_image_type = moving_image.GetPixelID()
    moving_image = sitk.Cast(moving_image, sitk.sitkFloat32)

    # (Optional) isotropic resample
    # This changes the behaviour, so care should be taken
    # For highly anisotropic images may be preferable

    if isotropic_resample:
        # First, copy the fixed image so we can resample back into this space at the end
        fixed_image_original = fixed_image
        fixed_image_original.MakeUnique()

        fixed_image = smooth_and_resample(
            fixed_image,
            isotropic_voxel_size_mm=initial_isotropic_size,
        )

        moving_image = smooth_and_resample(
            moving_image,
            isotropic_voxel_size_mm=initial_isotropic_size,
        )

    else:
        fixed_image_original = fixed_image

    # Set up image registration method
    registration = sitk.ImageRegistrationMethod()
    registration.SetNumberOfThreads(ncores)

    registration.SetShrinkFactorsPerLevel(resolution_staging)
    registration.SetSmoothingSigmasPerLevel(smooth_sigmas)
    registration.SmoothingSigmasAreSpecifiedInPhysicalUnitsOn()

    # Choose optimiser
    if optimiser.lower() == "lbfgsb":
        registration.SetOptimizerAsLBFGSB(
            gradientConvergenceTolerance=1e-5,
            numberOfIterations=number_of_iterations,
            maximumNumberOfCorrections=5,
            maximumNumberOfFunctionEvaluations=1024,
            costFunctionConvergenceFactor=1e7,
            trace=verbose,
        )
    elif optimiser.lower() == "lbfgs":
        registration.SetOptimizerAsLBFGS2(
            numberOfIterations=number_of_iterations,
            solutionAccuracy=1e-2,
            hessianApproximateAccuracy=6,
            deltaConvergenceDistance=0,
            deltaConvergenceTolerance=0.01,
            lineSearchMaximumEvaluations=40,
            lineSearchMinimumStep=1e-20,
            lineSearchMaximumStep=1e20,
            lineSearchAccuracy=0.01,
        )
    elif optimiser.lower() == "cgls":
        registration.SetOptimizerAsConjugateGradientLineSearch(
            learningRate=0.05, numberOfIterations=number_of_iterations
        )
        registration.SetOptimizerScalesFromPhysicalShift()
    elif optimiser.lower() == "gradient_descent":
        registration.SetOptimizerAsGradientDescent(
            learningRate=5.0,
            numberOfIterations=number_of_iterations,
            convergenceMinimumValue=1e-6,
            convergenceWindowSize=10,
        )
        registration.SetOptimizerScalesFromPhysicalShift()
    elif optimiser.lower() == "gradient_descent_line_search":
        registration.SetOptimizerAsGradientDescentLineSearch(
            learningRate=1.0, numberOfIterations=number_of_iterations
        )
        registration.SetOptimizerScalesFromPhysicalShift()

    # Set metric
    if metric == "correlation":
        registration.SetMetricAsCorrelation()
    elif metric == "mean_squares":
        registration.SetMetricAsMeanSquares()
    elif metric == "demons":
        registration.SetMetricAsDemons()
    elif metric == "mutual_information":
        registration.SetMetricAsMattesMutualInformation(
            numberOfHistogramBins=number_of_histogram_bins_mi
        )

    registration.SetInterpolator(sitk.sitkLinear)

    # Set sampling
    if isinstance(sampling_rate, float):
        registration.SetMetricSamplingPercentage(sampling_rate)
    elif type(sampling_rate) in [np.ndarray, list]:
        registration.SetMetricSamplingPercentagePerLevel(sampling_rate)

    registration.SetMetricSamplingStrategy(sitk.ImageRegistrationMethod.REGULAR)

    # Set masks
    if moving_structure is not False:
        registration.SetMetricMovingMask(moving_structure)

    if fixed_structure is not False:
        registration.SetMetricFixedMask(fixed_structure)

    # Set control point spacing
    transform_domain_mesh_size = control_point_spacing_distance_to_number(
        fixed_image, initial_grid_spacing
    )

    if verbose:
        print(f"Initial grid size: {transform_domain_mesh_size}")

    # Initialise transform
    initial_transform = sitk.BSplineTransformInitializer(
        fixed_image,
        transformDomainMeshSize=[int(i) for i in transform_domain_mesh_size],
    )
    registration.SetInitialTransformAsBSpline(
        initial_transform, inPlace=True, scaleFactors=grid_scale_factors
    )

    # (Optionally) add iteration commands
    if verbose:
        registration.AddCommand(
            sitk.sitkIterationEvent,
            lambda: registration_command_iteration(registration),
        )
        registration.AddCommand(
            sitk.sitkMultiResolutionIterationEvent,
            lambda: stage_iteration(registration),
        )

    # Run the registration
    output_transform = registration.Execute(fixed=fixed_image, moving=moving_image)

    # Resample moving image
    registered_image = apply_transform(
        input_image=moving_image,
        reference_image=fixed_image_original,
        transform=output_transform,
        default_value=default_value,
        interpolator=interp_order,
    )

    registered_image = sitk.Cast(registered_image, moving_image_type)

    dvf_image = sitk.TransformToDisplacementField(
        output_transform,
        sitk.sitkVectorFloat32,
        fixed_image_original.GetSize(),
        fixed_image_original.GetOrigin(),
        fixed_image_original.GetSpacing(),
        fixed_image_original.GetDirection(),
    )
    output_transform = sitk.DisplacementFieldTransform(dvf_image)

    # Return outputs
    return registered_image, output_transform



import numpy as np
import glob
import os

from pathlib import Path
from pydicom import dcmread
from pydicom.dataset import Dataset


def sort_ds_list(ds_list: list[Dataset]) -> list[Dataset]:
    return sorted(ds_list, key=get_slice_position, reverse=False)

def get_slice_position(series_slice: Dataset):
    _, _, slice_direction = get_slice_directions(series_slice)
    return np.dot(slice_direction, series_slice.ImagePositionPatient)

def get_slice_directions(series_slice: Dataset):
    orientation = series_slice.ImageOrientationPatient
    row_direction = np.array(orientation[:3])
    column_direction = np.array(orientation[3:])
    slice_direction = np.cross(row_direction, column_direction)
    if not np.allclose(
        np.dot(row_direction, column_direction), 0.0, atol=1e-3
    ) or not np.allclose(np.linalg.norm(slice_direction), 1.0, atol=1e-3):
        raise Exception("Invalid Image Orientation (Patient) attribute")
    return row_direction, column_direction, slice_direction

def load_sorted_image_series(
    image_series_path: str | list[str],
    ) -> list[Dataset]:
    """
    Load a sorted list of DICOM image series from a directory or a list of paths.
    Args:
        image_series_path (str | list[str]): The path to the directory containing DICOM images or a list of paths to DICOM files.
    Returns:
        list[Dataset]: A sorted list of DICOM datasets.
    """
    dcm_path_list = []
    if isinstance(image_series_path, str) and Path(image_series_path).is_dir():
        # Check if directory exists
        if not Path(image_series_path).exists():
            raise FileNotFoundError(f"Directory {image_series_path} does not exist")
        dcm_path_list = glob.glob(os.path.join(image_series_path, "*.dcm"))
    # Check if image_series_path is a list of strings
    if isinstance(image_series_path, list) and all(isinstance(path, str) for path in image_series_path):
        # Validate each path in the list exists
        for path in image_series_path:
            if not Path(path).exists():
                raise FileNotFoundError(f"File {path} does not exist")
        dcm_path_list = image_series_path

    if len(dcm_path_list) == 0:
        raise FileNotFoundError(f"No DICOM files found in {image_series_path}")

    image_ds_list = []
    try:
        for dcm_path in dcm_path_list:
            image_ds_list.append(dcmread(dcm_path))
    except Exception as e:
        raise ValueError(f"Error reading DICOM file {dcm_path}: {e}")

    image_ds_list = sort_ds_list(image_ds_list)
    return image_ds_list


if __name__ == "__main__":
    orientation = [-1, 0, 0, 0, -1, 0]
    row_direction = np.array(orientation[:3])
    column_direction = np.array(orientation[3:])
    slice_direction = np.cross(row_direction, column_direction)
    print(slice_direction)
    image_position = [2, 2, 2]
    slice_idx = np.dot(slice_direction, image_position)
    print(slice_idx)
    print(sorted([-1, -2, -3, -4], reverse=False))
